#include "expression.hpp"

const std::string binary_expression::type_name = "expression";
const std::string unary_expression::type_name = "expression";

binary_expression::binary_expression(symbol&& s1, const std::string& op, symbol&& s2) :
    s1(std::move(s1)), s2(std::move(s2)), op(op) {}

binary_expression* binary_expression::clone() const {
    return new binary_expression(s1.clone(), op, s2.clone());
}

symbol_impl* binary_expression::evaluate(const interpreter_context& context) const {
    try {
        return context.binary_operator(op, s1, s2);
    } catch (no_binary_operator_match_exception& e) {
        context.msg(
            "error: no match for operator '" + e.s1.type() + ' '
            + e.op + ' ' + e.s2.type() + "' in :\n\t" + serialize(context)
        );
        throw parser_exception();
    } catch (...) {
        throw;
    }
}

std::string binary_expression::serialize(const interpreter_context& context) const {
    size_t p = priority(context);
    std::string s;
    if (p < s1.priority(context))
        s += "(" + s1.serialize(context) + ")";
    else
        s += s1.serialize(context);
    s += op;
    if (p < s2.priority(context))
        s += "(" + s2.serialize(context) + ")";
    else
        s += s2.serialize(context);
    return s;
}

const std::string& binary_expression::type() const {
    return type_name;
}

bool binary_expression::is_litteral() const {
    return false;
}

size_t binary_expression::priority(const interpreter_context& context) const {
    return context.binary_operator_priority(op);
}




unary_expression::unary_expression(const std::string& op, symbol&& s) :
   s(std::move(s)), op(op) {}

unary_expression* unary_expression::clone() const {
    return new unary_expression(op, s.clone());
}

symbol_impl* unary_expression::evaluate(const interpreter_context& context) const {
    try {
        return context.unary_operator(op, s);
    } catch (no_unary_operator_match_exception& e) {
        context.msg(
            "error: no match for operator '" + e.op + e.s.type() +
            "' in :\n\t" + serialize(context)
        );
        throw parser_exception();
    } catch (...) {
        throw;
    }
}

std::string unary_expression::serialize(const interpreter_context& context) const {
    if (priority(context) < s.priority(context))
        return op + "(" + s.serialize(context) + ")";
    else
        return op + s.serialize(context);
}

const std::string& unary_expression::type() const {
    return type_name;
}

bool unary_expression::is_litteral() const {
    return false;
}

size_t unary_expression::priority(const interpreter_context& context) const {
    return context.unary_operator_priority(op);
}
